# Reading file inputs ----------------------------------------------------------

# Read gene expression data from file
loadExprFile <- reactive({
    inFile <- input$expr
    if (is.null(inFile))
      return(NULL)     
    #n <- nchar(inFile$name)
    #ext <- substr(inFile$name, n-3, n)D
    expr <- readExprTxtFile(inFile$datapath)
    return(expr)
})

# Read gene expression data from file
exprInput <- reactive({
    expr <- loadExprFile()
    if (is.null(input$annotationOptions))
        return(expr)
    if (input$annotationOptions == "nocollapse")
        return(expr)
    if (is.null(input$collapsingMethod))
        return(expr)
    name <- collapsingMethodsMatrix[input$collapsingMethod, "Options"]
    annotation <- annotationInput()
    if (is.null(annotation))
        return(expr)
    if (name == "")
        options=NULL
    else {
        if (is.null(input$collapsingMethodOptions))
            return(expr)
        name <- strsplit(name, "=")
        name <- name[[1]][1]
        name <- strsplit(name, " ")
        name <- name[[1]]
        newName <- tolower(name[1])
        n <- length(name)
        if (n > 1) {
            for (i in 2:n) {
                str <- name[i]
                first <- toupper(substr(str, 1, 1))
                tail <- ""
                if (nchar(str) > 1)
                    tail <- tolower(substr(str, 2, nchar(str)))
                newName <- paste(newName, first, tail, sep="")
            }
        }
        options <- list()
        options[newName] <- input$collapsingMethodOptions
    }
    method <- match.fun(collapsingMethodsMatrix[input$collapsingMethod, 
                                                "Function"])
    return(collapseExprData(expr, annotation, method, options))
})

# Read categorical class file
labelsInput <- reactive({
    inFile <- input$labels
    if (is.null(inFile))
      return(NULL)
    labels <- readClsFile(inFile$datapath)
    return(labels)
})

# Read gene set data file 
geneSetsInput <- reactive ({
    inFile <- input$geneSets
    if (is.null(inFile))
      return(NULL)
    geneSets <- readGmtFile(inFile$datapath)
    return(geneSets)
})  

# Read gene set data file 
annotationInput <- reactive ({
    inFile <- input$annotation
    if (is.null(inFile))
      return(NULL)
    return(readChipFile(inFile$datapath))
})

# Gene set properties ----------------------------------------------------------

# Vector of gene set sizes
geneSetSizes <- reactive({
    geneSets <- geneSetsInput()
    if (is.null(geneSets))
        return(NULL)
    sizes <- array(NA, length(geneSets))
    for (i in 1:length(geneSets)) {
        sizes[i] <- length(geneSets[[i]]) - 2
    }
    return(sizes)
})

# Vector of gene set sizes, tanking into account only the genes that are 
# present in the gene expression data 
geneSetSizesInChip <-  reactive({
    geneSets <- geneSetsInput()
    expr <- exprInput()
    if (is.null(geneSets) || is.null(expr))
        return(NULL)
    sizes <- array(NA, length(geneSets))
    for (i in 1:length(geneSets)) {
        genes <- which(geneSets[[i]] %in% rownames(expr))
        sizes[i] <- length(genes)
    }
    return(sizes)
})

# Min gene set size. If gene expression data is loaded, it takes into 
# account only the genes that are present in the gene expression data. 
# Otherwise, it considers all genes in the gene set file.
minSize <- reactive({
    if (is.null(exprInput())) {
        if (is.null(geneSetSizes()))
            return(NULL)
        return(min(geneSetSizes()))
    }
    if(is.null(geneSetSizesInChip()))
        return(NULL)
    return(min(geneSetSizesInChip()))
})

# Max gene set size. If gene expression data is loaded, it takes into 
# account only the genes that are present in the gene expression data. 
# Otherwise, it considers all genes in the gene set file.
maxSize <- reactive({
    if (is.null(exprInput())) {
        if (is.null(geneSetSizes()))
            return(NULL)
        return(max(geneSetSizes()))
    }
    if(is.null(geneSetSizesInChip()))
        return(NULL)
    return(max(geneSetSizesInChip()))
})

# Phenotype properties --------------------------------------------------------- 

# Returns a string containg information about the phenotype classes
classes <- reactive({
    labels <- labelsInput()
    symbols <- unique(labels[[3]])
    classes <- paste(labels[[2]][2], " (", length(which(
                     labels[[3]] == symbols[1])), " samples)", sep="")
    if (as.numeric(labels[[1]][2]) > 2)
        for (i in 3:as.numeric(labels[[1]][2])) {
            classes <- paste(classes, ", ", labels[[2]][i], " (", 
                       length(which(labels[[3]] == symbols[i-1])), 
                       " samples)", sep="")
        }
    i <- as.numeric(labels[[1]][2]) + 1
    classes <- paste(classes, " and ", labels[[2]][i], " (", 
                   length(which(labels[[3]] == symbols[i-1])), 
                   " samples).", sep="")
    return(classes)
})

# Returns a vector containing 0 for each class1 sample, 1 for each class2
# sample and -1 for the remaining samples. 
labels <- reactive({
    classes <- input$classes
    if (is.null(classes))
        return(NULL)
    classes <- strsplit(classes, " ")
    labels <- labelsInput()
    l <- labelsAux(labels, classes[[1]][1], classes[[1]][2])
    return(l)
})

# Rendering --------------------------------------------------------------------

# _____Side bar

# Select predefined gene sets
output$collapsingMethod <- renderUI({
    if (is.null(input$annotationOptions))
        return(NULL)
    if (input$annotationOptions == "nocollapse")
        return(NULL)
    selectInput("collapsingMethod", "Select a method to collapse the dataset to gene symbols:",
                 collapsingMethods)
})

output$collapsingMethodDescription <- renderUI({
    if (is.null(input$annotationOptions))
        return(NULL)
    if (input$annotationOptions == "nocollapse")
        return(NULL)
    if (is.null(input$collapsingMethod))
        return(NULL)
    description <- collapsingMethodsMatrix[input$collapsingMethod,"Description"]
    if (description == "")
        return(NULL)
    return(helpText(description))
})

output$collapsingMethodOptions <- renderUI({
    if (is.null(input$annotationOptions))
        return(NULL)
    if (input$annotationOptions == "nocollapse")
        return(NULL)
    if (is.null(input$collapsingMethod))
        return(NULL)
    options <- collapsingMethodsMatrix[input$collapsingMethod, "Options"]
    if (options == "")
        return(NULL)
    options <- strsplit(options, "=")
    name <- options[[1]][1]
    options <- strsplit(options[[1]][2], "#")
    description <- options[[1]][2]
    options <- strsplit(options[[1]][1], ",")
    options <- options[[1]]
    name <- paste(name, ":", sep="")
    if (!is.na(description))
        name <- p(name, img(src="images/info.png", title=description))
    selectInput("collapsingMethodOptions", name, options)
 })

# _____Data tab

# Render gene expression data information
output$expr <- renderUI({
  expr <- exprInput()
  if (is.null(expr) || is.null(input$expr))
      return("No gene expression data file was loaded.")
  else {
      dim <- dim(expr)
      filename <- paste("\"", input$expr$name, "\"", sep="")
      msg <- paste(filename, "was loaded successfully. The",
             "expression matrix has", dim[1], "rows (genes) and", dim[2], 
             "columns (samples).")
      return(msg)
  }
})

# Render phenotype data information
output$labels <- renderUI({
    labels <- labelsInput()
    if (is.null(labels) || is.null(input$labels))
        return("No categorical class file was loaded.")
    else {
        filename <- paste("\"", input$labels$name, "\"", sep="")
        classes <- classes()
        msg <- paste(filename, "was loaded successfully.", 
                    "The classes are: ", classes)
        return(msg)
    }
})

# Render gene sets information
output$geneSets <- renderUI({
    geneSets <- geneSetsInput()
    if (is.null(geneSets))
        return("No gene sets collection file was loaded.")
    else {
        filename <- paste("\"", input$geneSets$name, "\"", sep="")
        msg <- paste(filename, "was loaded successfully.", 
            "The collection contains", length(geneSets), 
            "gene sets ranging from")
        expr <- exprInput()
        if (is.null(expr)) {
            msg <- paste(msg, min(geneSetSizes()), "to", max(geneSetSizes()), 
                         "genes.")
        }
        else {
            msg <- paste(msg, " ", min(geneSetSizes()), " (", minSize(), 
                        " in the expression data) to ", max(geneSetSizes()), 
                        " (", maxSize(), " in the expression data) genes.", 
                        sep="")
        }
        return(msg)
    }
})

# Observing user inputs --------------------------------------------------------

# Select the data tab when a file is loaded
observe({
    input$expr
    input$geneSets
    input$labels
    updateTabsetPanel(session, "tabSelected", selected = "Loaded data")
})
