# Results computation ------------------------------------------------------

# Returns the analysis results matrix
results <- reactive ({
    results <- data.frame(matrix(NA, 1, ncol=5))
    colnames(results) <- c("Genes set name", "Set size", "Test statistic", 
                           "Nominal p-value", "Q-value")
    if (!values$canExecute || input$start==0)
        return(results)
    createAlert(session, inputId = "resultsWarning",
                message = paste("The analysis is running..."),
                type = "info",
                dismiss = TRUE,
                block = FALSE,
                append = FALSE)
    isolate({
        expr <- values$expr
        labels <- values$labels
        geneSets <- values$filteredGeneSets
        numPermutations <- values$numPermutations
        correlationMeasure <- values$correlationMeasure
        associationMeasure <- values$associationMeasure
        networkType <- values$networkType
        threshold <- values$threshold
        networkTest <- values$networkTest
        options <- values$networkTestOptions
        seed <- values$seed
        printParameters <- function(){print(values$parameters)}
        if (is.null(expr) || is.null(labels) || is.null(geneSets) ||
            is.null(numPermutations) || is.null(correlationMeasure))
            return(NULL)
        if (associationMeasure=="correlation")
            col <- 1
        else 
            col <- 2
        if (!is.null(options)) {
            name <- networkTestsMatrix[networkTest, "Options"]
            name <- strsplit(name, "=")[[1]][1]
            ops <- list()
            ops[tolower(name)] <- options
            options <- ops
        }
        networkInference <- 
                     match.fun(correlationMeasures[correlationMeasure, col])
        print <- F
        adjacencyMatrix <- adjacencyMatrix(networkInference, 
                                           T,
                                           ifelse(associationMeasure==
                                                  "correlation", F, T), 
                                           ifelse(associationMeasure=="qvalue", 
                                                  T, F),
                                           ifelse(networkType=="weighted", T, 
                                                  F), threshold)
        logFile=stdout()
        method <- match.fun(networkTestsMatrix[networkTest, 2])
        results <- data.frame(matrix(NA, nrow=length(geneSets), ncol=4))
        names <- array(NA, length(geneSets))
        for (i in 1:length(geneSets)) {
            names[i] <- geneSets[[i]][1]
        }
        rownames(results) <- names
        colnames(results) <- c("Set size", "Test statistic", "Nominal p-value", "Q-value")
        classes <- values$classes
        c1 <- classes[[1]][1]
        c2 <- classes[[1]][2]
        #temp <- tempfile(paste("CoGA_results_", c1, "_vs_", c2, sep=""), 
        #                 fileext=".txt")
        withProgress(session, min=1, max=length(geneSets), {
            setProgress(message = 'Analysis in progress',
                         detail = 'This may take a while...')
            for (i in 1:length(geneSets)) {
                setName <- geneSets[[i]][1]
                setProgress(value = i)
                msg <- paste("Testing ", setName, " (", i, " of ", length(geneSets), 
                              " sets)", sep="")
                setProgress(detail = msg)
                if (print)
                    cat(msg, file=logFile, append=T)
                genes <- geneSets[[i]][geneSets[[i]] %in% rownames(expr)]
                if (!is.null(seed))
                    set.seed(seed)
                result <- method(expr[genes, ], labels, adjacencyMatrix=
                                  adjacencyMatrix,  numPermutations=
                                  numPermutations, options=options)
                results[setName, "Test statistic"] <<- result[[1]]
                results[setName, "Nominal p-value"] <<- result$p.value
                results[setName, "Set size"] <<- length(genes)
                #if (!is.null(resultsFile))
                #save(results, printParameters, file=temp)
            }
        })
        results[, "Q-value"] <- p.adjust(results[, "Nominal p-value"], 
                                            method="fdr")
        results <- cbind(rownames(results), results)
        colnames(results)[1] <- "Genes set name" 
        return(results) 
    })
})

# Rendering --------------------------------------------------------------------

# _____Results tab

# Render application messages 
output$appMessages <- renderUI({
    if (input$start == 0) {
        return("")
    }
    isolate({
        expr <- exprInput()
        labels <- labelsInput()
        geneSets <-geneSetsInput()
        min <- input$minSize
        max <- input$maxSize
        numPermutations <- input$numPermutations
        correlationMeasure <- input$correlationMeasure
        associationMeasure <- input$associationMeasure
        networkTest <- input$networkTest
        networkType <- input$networkType
        threshold <- input$edgeThreshold
        values$canExecute <- F
        seed <- input$seed
        options <- input$networkTestOptions
        exprInputFileName <- input$expr$name
        geneSetsInputFileName <- input$geneSets$name
        labelsInputFileName <- input$labels$name
        if (is.null(expr)) {
            msg <- paste("To run the analysis, you must load the gene expression",
                         "data. Please, load the data on the side panel and",
                         "try again.")
            stop(msg)
        }
        if (is.null(labels)) {
            msg <- paste("To run the analysis, you must load the phenotype data.",
                         "Please, load the data on the side panel and",
                         "try again.")
            stop(msg)
        }
        if (is.null(geneSets)) {
            msg <- paste("To run the analysis, you must load the gene set data.",
                         "Please, load the data on the side panel and",
                         "try again.")
            stop(msg)
        }
        if (is.na(min)) {
            msg <- "Please, enter a valid minimum gene set size."
            stop(msg)
        }
        if (is.na(max)) {
            msg <- "Please, enter a valid maximum gene set size."
            stop(msg)
        }
        if (min > max) {
            msg <- paste("Minimum gene set size should be less than or",
                         "equal to the maximum gene set size.")
            stop(msg)
        }
        if (min < minSize()) {
            min <- minSize()
            updateNumericInput(session, "minSize", min)
        }
        if (max > maxSize()) {
            max <- maxSize()
            updateNumericInput(session, "maxSize", max) 
        }
        if (length(filteredGeneSets) == 0) {
            msg <- paste("There are no gene sets with sizes between", 
                          min, "and", max, ". Please, choose a new size",
                          "interval on the side panel and try to run",
                          "again.")
            stop(msg)
        }   
        if (minSize() < 2 && min < 2) {
            msg <- paste("All gene sets should contain at least 2 genes.", 
                         "Please, choose a new minimum size on the side", 
                         "panel and try again.")
            stop(msg)
        }
        if (numPermutations < 1) {
            msg <- paste("The number of permutations should be positive.")
            stop(msg)
        }

        geneSetDatabase <- paste("\"", geneSetsInputFileName, "\"", sep="")
        classes <- input$classes
        classes <- strsplit(classes, " ")
        l <- labels()
        names <- labels[[2]][2:length(labels[[2]])]
        i <- which(names == classes[[1]][1])
        j <- which(names == classes[[1]][2])
        symbols <- unique(labels[[3]])
        phen <- paste(names[i], " (", length(which(labels[[3]] == symbols[i])),
                      " samples) and ", names[j], " (", 
                      length(which(labels[[3]] == symbols[j])), " samples)", 
                      sep="")
        associationMsg <- c("correlation"="absolute correlation",
                            "pvalue"="1 - pvalue", "qvalue"="1 - qvalue")
        if (is.null(seed))
          seedMsg <- "created from the current time and the process ID"
        else 
          seedMsg <- seed

        if (networkTestsMatrix[networkTest, "Options"] == "")
          options <- NULL
        if (is.null(options))
          optionsMsg <- ""
        else { 
          name <- networkTestsMatrix[networkTest, "Options"]
          name <- strsplit(name, "=")[[1]][1]
          optionsMsg <- paste("*", name, "-", options, "\n")
        }
        thresholdMsg <- ""
        if (networkType == "unweighted")
          thresholdMsg <- paste("* Association degree threshold for edge", 
                                "selection -", threshold, "\n")

        parameters <- paste("* Expression data from \"", exprInputFileName, "\"", 
                  " - ", nrow(expr)," genes and ", ncol(expr), " samples",
                  "\n", "* Phenotype data from \"", labelsInputFileName, 
                  "\" - ", phen, "\n", 
                  "* Gene sets from ", geneSetDatabase, " - ", 
                  length(filteredGeneSets()), " filtered gene sets.", 
                  " The set sizes vary between ", min, " and ", 
                  max, "\n",
                  "* Network type - ", networkType, "\n",
                  "* Association measure - ", correlationMeasure, " (", 
                  associationMsg[associationMeasure], ")", "\n", 
                  thresholdMsg,
                  "* Method for networks comparison - ", networkTest, "\n",
                  optionsMsg, 
                  "* Number of label permutations - ", input$numPermutations,
                  "\n",  
                  "* Seed used to generate random permutations - ", 
                  seedMsg, "\n\n",  sep="")
        msg <- paste("The analysis is running with the following parameters:", 
                  "\n\n", parameters, sep="")
        cat(msg)
        if (optionsMsg != "")
          optionsMsg <- div(optionsMsg, br())
        if (thresholdMsg != "")
          thresholdMsg <- div(thresholdMsg, br())
        msg <- p(h5("Execution parameters"),
                  "* Expression data from \"", exprInputFileName, "\"", 
                  " - ", nrow(expr)," genes and ", ncol(expr), " samples",
                  br(), "* Phenotype data from \"", labelsInputFileName, 
                  "\" - ", phen, br(), 
                  "* Gene sets from ", geneSetDatabase, " - ", 
                  length(filteredGeneSets()), " filtered gene sets.", 
                  " The set sizes vary between ", min, " and ", 
                  max, br(),
                  "* Network type - ", networkType, br(),
                  "* Association measure - ", correlationMeasure, " (", 
                  associationMsg[associationMeasure], ")", br(),
                  thresholdMsg,
                  "* Method for networks comparison - ", networkTest, br(),
                  optionsMsg,
                  "* Number of label permutations - ", input$numPermutations,
                  br(),
                  "* Seed used to generate random permutations - ", 
                  seedMsg)
        values$expr <- expr
        values$filteredGeneSets <- geneSets[filteredGeneSets()]
        values$labels <- l
        values$numPermutations <- numPermutations
        values$completed <- F
        values$canExecute <- T
        values$classes <- classes
        values$correlationMeasure <- correlationMeasure
        values$associationMeasure <- associationMeasure
        values$networkTest <- networkTest
        values$networkType <- networkType
        values$threshold <- threshold
        values$seed <- seed
        values$networkTestOptions <- options
        values$parameters <- parameters
        values$exprInputFileName <- exprInputFileName
        values$labelsInputFileName <- labelsInputFileName
        values$geneSetsInputFileName <- geneSetsInputFileName
        return(msg)
    })
})

# Render table results
#output$results <- renderDataTable({
#    if (!values$completed)
#      return(NULL)
#    isolate({
#      return(results())
#    })}, options=list(aoColumns = list(list(bSearchable = FALSE), 
#                                       list(bSearchable = FALSE),
#                                       list(bSearchable = FALSE), 
#                                       list(bSearchable = FALSE),
#                                       list(bSearchable = FALSE))))



output$results <- renderChart2({
      table <- results()
      colnames(table)[5] <- paste("Q-value <img src=\"images/info.png\" ", 
                                  "title=\"Adjusted p-value for ",
                                  "multiple comparisons (Benjamin and ",
                                  "Hochberg FDR method)\" />", sep="")
      return(dTable(table))
})

# Render radio buttons that show the result file format options. 
output$resultsType <- renderUI({
    if (!values$completed) {
        return(NULL)
    }
    radioButtons("resultsType", "Select a file format to save the results:",
                  c("CSV", "R data"))

})

# Render download results button
output$downloadResultsButton <- renderUI({
    if (!values$completed)
        return(NULL)
    downloadButton("downloadResults", "Save results")
})

# Prepare results file for download 
output$downloadResults <- downloadHandler(
    filename = function() {
        classes <- values$classes
        c1 <- classes[[1]][1]
        c2 <- classes[[1]][2]
        if (input$resultsType == "R data")
            name <- paste("CoGA_results_", c1, "_vs_", c2, ".RData", sep="")
        else 
            name <- paste("CoGA_results_", c1, "_vs_", c2, ".csv", sep="")
        return(name)
    },
    content = function(filename) {
        results <- results()
        rownames(results) <- NULL
        parameters <- matrix(NA, 18, ncol(results))
        expr <- values$expr
        classes <- values$classes
        c1 <- classes[[1]][1]
        c2 <- classes[[1]][2]
        labels <- values$labels
        associationMsg <- c("correlation"="absolute correlation",
                            "pvalue"="1 - pvalue", "qvalue"="1 - qvalue")
        options <- values$networkTestOptions
        if (is.null(values$seed))
          seedMsg <- "created from the current time and the process ID"
        else 
          seedMsg <- values$seed

        if (networkTestsMatrix[values$networkTest, "Options"] == "")
          options <- NULL
        if (is.null(options))
          optionsMsg <- ""
        else { 
          name <- networkTestsMatrix[values$networkTest, "Options"]
          name <- strsplit(name, "=")[[1]][1]
          optionsMsg <- paste(tolower(name), "-", options)
        }
        thresholdMsg <- "none (full graph)"
        if (values$networkType == "unweighted")
          thresholdMsg <- values$threshold
        parameters[1, 1] <- "CoGA differential network analysis"
        parameters[2, 1] <- "Date:"
        parameters[2, 2] <- date()
        parameters[3, 1] <- "Gene expression input file:"
        parameters[3, 2] <- values$exprInputFileName
        parameters[4, 1] <- "Number of genes:" 
        parameters[4, 2] <- nrow(expr)
        parameters[5, 1] <- "Number of samples:" 
        parameters[5, 2] <- ncol(expr)
        parameters[6, 1] <- "Sample labels input file:"
        parameters[6, 2] <- values$labelsInputFileName
        parameters[7, 1] <- paste("Number of samples from class ", c1, ":", 
                                  sep="")
        parameters[7, 2] <- length(which(labels == 0))
        parameters[8, 1] <- paste("Number of samples from class ", c2, ":", 
                                  sep="")
        parameters[8, 2] <- length(which(labels == 1))
        parameters[9, 1] <- "Gene sets collection:"
        parameters[9, 2] <- values$geneSetsInputFileName
        parameters[10, 1] <- "Number of tested gene sets:"
        parameters[10, 2] <- length(values$filteredGeneSets)
        parameters[11, 1] <- "Minimum gene set size:"
        parameters[11, 2] <-  min(as.numeric(lapply(values$filteredGeneSets, 
                                                    length)))
        parameters[12, 1] <- "Maximum gene set size:"
        parameters[12, 2] <-  max(as.numeric(lapply(values$filteredGeneSets, 
                                                    length)))
        parameters[13, 1] <- "Network type:"
        parameters[13, 2] <- values$networkType
        parameters[14, 1] <- "Association measure:"
        parameters[14, 2] <- paste(values$correlationMeasure, " (", 
                                   associationMsg[values$associationMeasure], 
                                   ")", sep="")
        parameters[15, 1] <- "Association degree threshold for edge selection:"
        parameters[15, 2] <- thresholdMsg
        parameters[16, 1] <- "Method for networks comparison:"
        parameters[16, 2] <- values$networkTest
        if (!is.null(options))
          parameters[16, 2] <- paste(parameters[16, 2], " (", optionsMsg, ")",
                                     sep="")
        parameters[17, 1] <- "Number of label permutations:"
        parameters[17, 2] <- values$numPermutations
        parameters[18, 1] <- "Seed used to generate random permutations:"
        parameters[18, 2] <- seedMsg


        if (input$resultsType == "R data") {
            parameters <- parameters[,1:2]
            parameters <- as.table(parameters)
            rownames(parameters) <- NULL
            colnames(parameters) <- NULL
            save(results, parameters, file=filename)
        }
        else {
            parameters <- rbind(parameters, rep(NA, ncol(results)))
            parameters <- rbind(parameters, rep(NA, ncol(results)))
            parameters <- rbind(parameters, colnames(results))
            write.table(parameters, filename, na="", col.names=F, row.names=F, 
                        sep=",")
            write.table(results, filename, append=T, row.names=F, col.names=F, 
                        sep=",", dec=".")
        }
    }
)

# Render message when GNEA analysis completes
output$isCompleted <- renderUI({
    if (!values$completed || input$start==0)
      return(NULL)
    isolate({
        msg <- "The analysis completed successfully."
        if (!is.null(results)) {
          cat(paste("\n", msg, "\n", sep=""))
          updateTabsetPanel(session, "tabSelected", 
                            selected="Analysis results")
          return(h5("Results"))
        }
        else
          return(NULL)
      })
})

# Observing user inputs --------------------------------------------------------

# Verifies when it must run GNEA
observe({
    values$canExecute
    if (values$canExecute) {
        results()
        if (!is.null(results)) {
            values$completed <- T
        }
    }
})


# Select the application messages tab when the user clicks on the Start GNEA
# button
observe({
    input$start
    if (input$start != 0)
    updateTabsetPanel(session, "tabSelected", selected = 
                      "Analysis results")
})

# Alerts -----------------------------------------------------------------------

createAlert(session, inputId = "resultsWarning",
            message = paste("The analysis is not running. To start, load the",
                            "data, set the execution parameters, and then",
                            "click on the \"Start analysis\" button on the",
                            "sidebar."),
            type = "info",
            dismiss = TRUE,
            block = FALSE,
            append = FALSE)

observe({
    if (values$completed) {
        r <- results()
        if (sum(is.na(r)) > 0) {
            createAlert(session, inputId = "resultsWarning",
                message = paste("There are missing results in your analysis.",
                                "Please, check out the \"Help\" section on", 
                                "\"Interpreting results\" to know", 
                                "more about missing p-values."),
                type = "warning",
                dismiss = TRUE,
                block = FALSE,
                append = FALSE
            )
        }
        else {
            createAlert(session, inputId = "resultsWarning",
                  message = paste("The analysis completed successfully."),
                  type = "success",
                  dismiss = TRUE,
                  block = FALSE,
                  append = FALSE
              )
        }
    }
    #else {
    #    closeAlert(session, "resultsWarning")
    #}
})
